/* menu_in_toolbar.c generated by valac 0.10.3, the Vala compiler
 * generated from menu_in_toolbar.vala, do not modify */

/*
 * This file is part of LaTeXila.
 *
 * Copyright © 2010 Sébastien Wilmet
 *
 * LaTeXila is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * LaTeXila is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with LaTeXila.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>


#define TYPE_MENU_TOOL_ACTION (menu_tool_action_get_type ())
#define MENU_TOOL_ACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MENU_TOOL_ACTION, MenuToolAction))
#define MENU_TOOL_ACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MENU_TOOL_ACTION, MenuToolActionClass))
#define IS_MENU_TOOL_ACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MENU_TOOL_ACTION))
#define IS_MENU_TOOL_ACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MENU_TOOL_ACTION))
#define MENU_TOOL_ACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MENU_TOOL_ACTION, MenuToolActionClass))

typedef struct _MenuToolAction MenuToolAction;
typedef struct _MenuToolActionClass MenuToolActionClass;
typedef struct _MenuToolActionPrivate MenuToolActionPrivate;

struct _MenuToolAction {
	GtkAction parent_instance;
	MenuToolActionPrivate * priv;
};

struct _MenuToolActionClass {
	GtkActionClass parent_class;
};


static gpointer menu_tool_action_parent_class = NULL;

GType menu_tool_action_get_type (void) G_GNUC_CONST;
enum  {
	MENU_TOOL_ACTION_DUMMY_PROPERTY
};
MenuToolAction* menu_tool_action_new (const char* name, const char* label, const char* tooltip, const char* stock_id);
MenuToolAction* menu_tool_action_construct (GType object_type, const char* name, const char* label, const char* tooltip, const char* stock_id);



MenuToolAction* menu_tool_action_construct (GType object_type, const char* name, const char* label, const char* tooltip, const char* stock_id) {
	MenuToolAction * self = NULL;
	GtkActionClass* ac;
	g_return_val_if_fail (name != NULL, NULL);
	self = (MenuToolAction*) g_object_new (object_type, "name", name, "label", label, "tooltip", tooltip, "stock-id", stock_id, NULL);
	ac = (GtkActionClass*) G_OBJECT_GET_CLASS ((GObject*) self);
	ac->toolbar_item_type = GTK_TYPE_MENU_TOOL_BUTTON;
	return self;
}


MenuToolAction* menu_tool_action_new (const char* name, const char* label, const char* tooltip, const char* stock_id) {
	return menu_tool_action_construct (TYPE_MENU_TOOL_ACTION, name, label, tooltip, stock_id);
}


static void menu_tool_action_class_init (MenuToolActionClass * klass) {
	menu_tool_action_parent_class = g_type_class_peek_parent (klass);
}


static void menu_tool_action_instance_init (MenuToolAction * self) {
}


GType menu_tool_action_get_type (void) {
	static volatile gsize menu_tool_action_type_id__volatile = 0;
	if (g_once_init_enter (&menu_tool_action_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (MenuToolActionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) menu_tool_action_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MenuToolAction), 0, (GInstanceInitFunc) menu_tool_action_instance_init, NULL };
		GType menu_tool_action_type_id;
		menu_tool_action_type_id = g_type_register_static (GTK_TYPE_ACTION, "MenuToolAction", &g_define_type_info, 0);
		g_once_init_leave (&menu_tool_action_type_id__volatile, menu_tool_action_type_id);
	}
	return menu_tool_action_type_id__volatile;
}




