/* symbols.c generated by valac 0.10.3, the Vala compiler
 * generated from symbols.vala, do not modify */

/*
 * This file is part of LaTeXila.
 *
 * Copyright © 2010 Sébastien Wilmet
 *
 * LaTeXila is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * LaTeXila is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with LaTeXila.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <config.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <stdio.h>
#include <gio/gio.h>
#include <gtksourceview/gtksourceview.h>
#include <gee.h>


#define TYPE_SYMBOLS (symbols_get_type ())
#define SYMBOLS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SYMBOLS, Symbols))
#define SYMBOLS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SYMBOLS, SymbolsClass))
#define IS_SYMBOLS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SYMBOLS))
#define IS_SYMBOLS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SYMBOLS))
#define SYMBOLS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SYMBOLS, SymbolsClass))

typedef struct _Symbols Symbols;
typedef struct _SymbolsClass SymbolsClass;
typedef struct _SymbolsPrivate SymbolsPrivate;

#define TYPE_MAIN_WINDOW (main_window_get_type ())
#define MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MAIN_WINDOW, MainWindow))
#define MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAIN_WINDOW, MainWindowClass))
#define IS_MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MAIN_WINDOW))
#define IS_MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAIN_WINDOW))
#define MAIN_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAIN_WINDOW, MainWindowClass))

typedef struct _MainWindow MainWindow;
typedef struct _MainWindowClass MainWindowClass;

#define SYMBOLS_TYPE_SYMBOL_COLUMN (symbols_symbol_column_get_type ())

#define SYMBOLS_TYPE_CATEGORY_COLUMN (symbols_category_column_get_type ())

#define SYMBOLS_TYPE_CATEGORY_INFO (symbols_category_info_get_type ())
typedef struct _SymbolsCategoryInfo SymbolsCategoryInfo;

#define SYMBOLS_TYPE_SYMBOL_INFO (symbols_symbol_info_get_type ())
typedef struct _SymbolsSymbolInfo SymbolsSymbolInfo;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define TYPE_APP_SETTINGS (app_settings_get_type ())
#define APP_SETTINGS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_SETTINGS, AppSettings))
#define APP_SETTINGS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_SETTINGS, AppSettingsClass))
#define IS_APP_SETTINGS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_SETTINGS))
#define IS_APP_SETTINGS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_SETTINGS))
#define APP_SETTINGS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_SETTINGS, AppSettingsClass))

typedef struct _AppSettings AppSettings;
typedef struct _AppSettingsClass AppSettingsClass;
#define _gtk_tree_path_free0(var) ((var == NULL) ? NULL : (var = (gtk_tree_path_free (var), NULL)))
#define __g_list_free_gtk_tree_path_free0(var) ((var == NULL) ? NULL : (var = (_g_list_free_gtk_tree_path_free (var), NULL)))

#define TYPE_DOCUMENT_TAB (document_tab_get_type ())
#define DOCUMENT_TAB(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DOCUMENT_TAB, DocumentTab))
#define DOCUMENT_TAB_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DOCUMENT_TAB, DocumentTabClass))
#define IS_DOCUMENT_TAB(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DOCUMENT_TAB))
#define IS_DOCUMENT_TAB_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DOCUMENT_TAB))
#define DOCUMENT_TAB_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DOCUMENT_TAB, DocumentTabClass))

typedef struct _DocumentTab DocumentTab;
typedef struct _DocumentTabClass DocumentTabClass;

#define TYPE_DOCUMENT (document_get_type ())
#define DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DOCUMENT, Document))
#define DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DOCUMENT, DocumentClass))
#define IS_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DOCUMENT))
#define IS_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DOCUMENT))
#define DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DOCUMENT, DocumentClass))

typedef struct _Document Document;
typedef struct _DocumentClass DocumentClass;

#define TYPE_DOCUMENT_VIEW (document_view_get_type ())
#define DOCUMENT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DOCUMENT_VIEW, DocumentView))
#define DOCUMENT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DOCUMENT_VIEW, DocumentViewClass))
#define IS_DOCUMENT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DOCUMENT_VIEW))
#define IS_DOCUMENT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DOCUMENT_VIEW))
#define DOCUMENT_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DOCUMENT_VIEW, DocumentViewClass))

typedef struct _DocumentView DocumentView;
typedef struct _DocumentViewClass DocumentViewClass;
#define _g_free0(var) (var = (g_free (var), NULL))
typedef struct _Block3Data Block3Data;

#define TYPE_MOST_USED_SYMBOL (most_used_symbol_get_type ())
typedef struct _MostUsedSymbol MostUsedSymbol;
#define _most_used_symbol_free0(var) ((var == NULL) ? NULL : (var = (most_used_symbol_free (var), NULL)))

struct _Symbols {
	GtkVBox parent_instance;
	SymbolsPrivate * priv;
};

struct _SymbolsClass {
	GtkVBoxClass parent_class;
};

struct _SymbolsPrivate {
	MainWindow* main_window;
};

typedef enum  {
	SYMBOLS_SYMBOL_COLUMN_PIXBUF,
	SYMBOLS_SYMBOL_COLUMN_COMMAND,
	SYMBOLS_SYMBOL_COLUMN_TOOLTIP,
	SYMBOLS_SYMBOL_COLUMN_ID,
	SYMBOLS_SYMBOL_COLUMN_PACKAGE,
	SYMBOLS_SYMBOL_COLUMN_N_COLUMNS
} SymbolsSymbolColumn;

typedef enum  {
	SYMBOLS_CATEGORY_COLUMN_ICON,
	SYMBOLS_CATEGORY_COLUMN_NAME,
	SYMBOLS_CATEGORY_COLUMN_N_COLUMNS
} SymbolsCategoryColumn;

struct _SymbolsCategoryInfo {
	char* name;
	char* icon;
};

struct _SymbolsSymbolInfo {
	char* filename;
	char* latex_command;
	char* package_required;
};

struct _Block3Data {
	int _ref_count_;
	Symbols * self;
	GtkIconView* categories_view;
	GtkIconView* symbol_view;
	GtkButton* button;
};

struct _MostUsedSymbol {
	char* id;
	char* latex_command;
	char* package_required;
	guint num;
};


static gboolean symbols_stores_initialized;
static gboolean symbols_stores_initialized = FALSE;
static GtkListStore* symbols_categories_store;
static GtkListStore* symbols_categories_store = NULL;
static GtkListStore** symbols_symbols_stores;
static gint symbols_symbols_stores_length1;
static GtkListStore** symbols_symbols_stores = NULL;
static gint symbols_symbols_stores_length1 = 0;
static gint _symbols_symbols_stores_size_ = 0;
static GtkListStore* symbols_mus_store;
static GtkListStore* symbols_mus_store = NULL;
static gpointer symbols_parent_class = NULL;

GType symbols_get_type (void) G_GNUC_CONST;
GType main_window_get_type (void) G_GNUC_CONST;
#define SYMBOLS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SYMBOLS, SymbolsPrivate))
enum  {
	SYMBOLS_DUMMY_PROPERTY
};
static GType symbols_symbol_column_get_type (void) G_GNUC_UNUSED;
static GType symbols_category_column_get_type (void) G_GNUC_UNUSED;
static GType symbols_category_info_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SymbolsCategoryInfo* symbols_category_info_dup (const SymbolsCategoryInfo* self);
static void symbols_category_info_free (SymbolsCategoryInfo* self);
static void symbols_category_info_copy (const SymbolsCategoryInfo* self, SymbolsCategoryInfo* dest);
static void symbols_category_info_destroy (SymbolsCategoryInfo* self);
static GType symbols_symbol_info_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static SymbolsSymbolInfo* symbols_symbol_info_dup (const SymbolsSymbolInfo* self);
static void symbols_symbol_info_free (SymbolsSymbolInfo* self);
static void symbols_symbol_info_copy (const SymbolsSymbolInfo* self, SymbolsSymbolInfo* dest);
static void symbols_symbol_info_destroy (SymbolsSymbolInfo* self);
Symbols* symbols_new (MainWindow* main_window);
Symbols* symbols_construct (GType object_type, MainWindow* main_window);
GdkPixbuf* utils_get_pixbuf_from_stock (const char* stock_id, GtkIconSize size);
static GtkListStore* symbols_get_symbol_store (Symbols* self, SymbolsSymbolInfo* symbols, int symbols_length1);
void symbols_reload_most_used_symbols (void);
static void symbols_create_icon_views (Symbols* self);
GtkWidget* utils_add_scrollbar (GtkWidget* child);
static void _lambda48_ (Symbols* self);
GType app_settings_get_type (void) G_GNUC_CONST;
AppSettings* app_settings_get_default (void);
void app_settings_clear_most_used_symbols (AppSettings* self);
static void __lambda48__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _lambda49_ (Block3Data* _data3_);
static void _g_list_free_gtk_tree_path_free (GList* self);
static void __lambda49__gtk_icon_view_selection_changed (GtkIconView* _sender, gpointer self);
static void _lambda50_ (Block3Data* _data3_);
GType document_tab_get_type (void) G_GNUC_CONST;
DocumentTab* main_window_get_active_tab (MainWindow* self);
GType document_get_type (void) G_GNUC_CONST;
Document* main_window_get_active_document (MainWindow* self);
GType document_view_get_type (void) G_GNUC_CONST;
DocumentView* main_window_get_active_view (MainWindow* self);
void app_settings_add_symbol (AppSettings* self, const char* id, const char* command, const char* package);
static void __lambda50__gtk_icon_view_selection_changed (GtkIconView* _sender, gpointer self);
static Block3Data* block3_data_ref (Block3Data* _data3_);
static void block3_data_unref (Block3Data* _data3_);
static void symbols_insert_symbol (GtkListStore* symbol_store, gint index, SymbolsSymbolInfo* symbol);
static char* symbols_get_symbol_id (const char* path);
GType most_used_symbol_get_type (void) G_GNUC_CONST;
MostUsedSymbol* most_used_symbol_dup (const MostUsedSymbol* self);
void most_used_symbol_free (MostUsedSymbol* self);
void most_used_symbol_copy (const MostUsedSymbol* self, MostUsedSymbol* dest);
void most_used_symbol_destroy (MostUsedSymbol* self);
GeeList* app_settings_get_most_used_symbols (AppSettings* self);
static void symbols_get_symbol_info_from_most_used (MostUsedSymbol* mus, SymbolsSymbolInfo* result);
void symbols_insert_most_used_symbol (gint index, MostUsedSymbol* symbol);
void symbols_remove_most_used_symbol (gint index);
void symbols_swap_most_used_symbol (gint current_index, gint new_index);
static void symbols_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const SymbolsCategoryInfo SYMBOLS_categories[7] = {{N_ ("Greek"), DATA_DIR "/images/icons/symbol_greek.png"}, {N_ ("Arrows"), DATA_DIR "/images/icons/symbol_arrows.png"}, {N_ ("Relations"), DATA_DIR "/images/icons/symbol_relations.png"}, {N_ ("Operators"), DATA_DIR "/images/icons/symbol_operators.png"}, {N_ ("Delimiters"), DATA_DIR "/images/icons/symbol_delimiters.png"}, {N_ ("Misc math"), DATA_DIR "/images/icons/symbol_misc_math.png"}, {N_ ("Misc text"), DATA_DIR "/images/icons/symbol_misc_text.png"}};
static const SymbolsSymbolInfo SYMBOLS_symbols_greek[51] = {{DATA_DIR "/images/greek/01.png", "\\alpha", NULL}, {DATA_DIR "/images/greek/02.png", "\\beta", NULL}, {DATA_DIR "/images/greek/03.png", "\\gamma", NULL}, {DATA_DIR "/images/greek/04.png", "\\delta", NULL}, {DATA_DIR "/images/greek/05.png", "\\epsilon", NULL}, {DATA_DIR "/images/greek/06.png", "\\varepsilon", NULL}, {DATA_DIR "/images/greek/07.png", "\\zeta", NULL}, {DATA_DIR "/images/greek/08.png", "\\eta", NULL}, {DATA_DIR "/images/greek/09.png", "\\theta", NULL}, {DATA_DIR "/images/greek/10.png", "\\vartheta", NULL}, {DATA_DIR "/images/greek/11.png", "\\iota", NULL}, {DATA_DIR "/images/greek/12.png", "\\kappa", NULL}, {DATA_DIR "/images/greek/13.png", "\\lambda", NULL}, {DATA_DIR "/images/greek/14.png", "\\mu", NULL}, {DATA_DIR "/images/greek/15.png", "\\nu", NULL}, {DATA_DIR "/images/greek/16.png", "\\xi", NULL}, {DATA_DIR "/images/greek/18.png", "\\pi", NULL}, {DATA_DIR "/images/greek/19.png", "\\varpi", NULL}, {DATA_DIR "/images/greek/20.png", "\\rho", NULL}, {DATA_DIR "/images/greek/21.png", "\\varrho", NULL}, {DATA_DIR "/images/greek/22.png", "\\sigma", NULL}, {DATA_DIR "/images/greek/23.png", "\\varsigma", NULL}, {DATA_DIR "/images/greek/24.png", "\\tau", NULL}, {DATA_DIR "/images/greek/25.png", "\\upsilon", NULL}, {DATA_DIR "/images/greek/26.png", "\\phi", NULL}, {DATA_DIR "/images/greek/27.png", "\\varphi", NULL}, {DATA_DIR "/images/greek/28.png", "\\chi", NULL}, {DATA_DIR "/images/greek/29.png", "\\psi", NULL}, {DATA_DIR "/images/greek/30.png", "\\omega", NULL}, {DATA_DIR "/images/greek/33.png", "\\Gamma", NULL}, {DATA_DIR "/images/greek/34.png", "\\varGamma", "amsmath"}, {DATA_DIR "/images/greek/35.png", "\\Delta", NULL}, {DATA_DIR "/images/greek/36.png", "\\varDelta", "amsmath"}, {DATA_DIR "/images/greek/40.png", "\\Theta", NULL}, {DATA_DIR "/images/greek/41.png", "\\varTheta", "amsmath"}, {DATA_DIR "/images/greek/44.png", "\\Lambda", NULL}, {DATA_DIR "/images/greek/45.png", "\\varLambda", "amsmath"}, {DATA_DIR "/images/greek/48.png", "\\Xi", NULL}, {DATA_DIR "/images/greek/49.png", "\\varXi", "amsmath"}, {DATA_DIR "/images/greek/51.png", "\\Pi", NULL}, {DATA_DIR "/images/greek/52.png", "\\varPi", "amsmath"}, {DATA_DIR "/images/greek/54.png", "\\Sigma", NULL}, {DATA_DIR "/images/greek/55.png", "\\varSigma", "amsmath"}, {DATA_DIR "/images/greek/57.png", "\\Upsilon", NULL}, {DATA_DIR "/images/greek/58.png", "\\varUpsilon", "amsmath"}, {DATA_DIR "/images/greek/59.png", "\\Phi", NULL}, {DATA_DIR "/images/greek/60.png", "\\varPhi", "amsmath"}, {DATA_DIR "/images/greek/62.png", "\\Psi", NULL}, {DATA_DIR "/images/greek/63.png", "\\varPsi", "amsmath"}, {DATA_DIR "/images/greek/64.png", "\\Omega", NULL}, {DATA_DIR "/images/greek/65.png", "\\varOmega", "amsmath"}};
static const SymbolsSymbolInfo SYMBOLS_symbols_arrows[69] = {{DATA_DIR "/images/arrows/01.png", "\\leftarrow", NULL}, {DATA_DIR "/images/arrows/02.png", "\\leftrightarrow", NULL}, {DATA_DIR "/images/arrows/03.png", "\\rightarrow", NULL}, {DATA_DIR "/images/arrows/04.png", "\\mapsto", NULL}, {DATA_DIR "/images/arrows/05.png", "\\longleftarrow", NULL}, {DATA_DIR "/images/arrows/06.png", "\\longleftrightarrow", NULL}, {DATA_DIR "/images/arrows/07.png", "\\longrightarrow", NULL}, {DATA_DIR "/images/arrows/08.png", "\\longmapsto", NULL}, {DATA_DIR "/images/arrows/09.png", "\\downarrow", NULL}, {DATA_DIR "/images/arrows/10.png", "\\updownarrow", NULL}, {DATA_DIR "/images/arrows/11.png", "\\uparrow", NULL}, {DATA_DIR "/images/arrows/12.png", "\\nwarrow", NULL}, {DATA_DIR "/images/arrows/13.png", "\\searrow", NULL}, {DATA_DIR "/images/arrows/14.png", "\\nearrow", NULL}, {DATA_DIR "/images/arrows/15.png", "\\swarrow", NULL}, {DATA_DIR "/images/arrows/16.png", "\\textdownarrow", "textcomp"}, {DATA_DIR "/images/arrows/17.png", "\\textuparrow", "textcomp"}, {DATA_DIR "/images/arrows/18.png", "\\textleftarrow", "textcomp"}, {DATA_DIR "/images/arrows/19.png", "\\textrightarrow", "textcomp"}, {DATA_DIR "/images/arrows/20.png", "\\nleftarrow", "amssymb"}, {DATA_DIR "/images/arrows/21.png", "\\nleftrightarrow", "amssymb"}, {DATA_DIR "/images/arrows/22.png", "\\nrightarrow", "amssymb"}, {DATA_DIR "/images/arrows/23.png", "\\hookleftarrow", NULL}, {DATA_DIR "/images/arrows/24.png", "\\hookrightarrow", NULL}, {DATA_DIR "/images/arrows/25.png", "\\twoheadleftarrow", "amssymb"}, {DATA_DIR "/images/arrows/26.png", "\\twoheadrightarrow", "amssymb"}, {DATA_DIR "/images/arrows/27.png", "\\leftarrowtail", "amssymb"}, {DATA_DIR "/images/arrows/28.png", "\\rightarrowtail", "amssymb"}, {DATA_DIR "/images/arrows/29.png", "\\Leftarrow", NULL}, {DATA_DIR "/images/arrows/30.png", "\\Leftrightarrow", NULL}, {DATA_DIR "/images/arrows/31.png", "\\Rightarrow", NULL}, {DATA_DIR "/images/arrows/32.png", "\\Longleftarrow", NULL}, {DATA_DIR "/images/arrows/33.png", "\\Longleftrightarrow", NULL}, {DATA_DIR "/images/arrows/34.png", "\\Longrightarrow", NULL}, {DATA_DIR "/images/arrows/35.png", "\\Updownarrow", NULL}, {DATA_DIR "/images/arrows/36.png", "\\Uparrow", NULL}, {DATA_DIR "/images/arrows/37.png", "\\Downarrow", NULL}, {DATA_DIR "/images/arrows/38.png", "\\nLeftarrow", "amssymb"}, {DATA_DIR "/images/arrows/39.png", "\\nLeftrightarrow", "amssymb"}, {DATA_DIR "/images/arrows/40.png", "\\nRightarrow", "amssymb"}, {DATA_DIR "/images/arrows/41.png", "\\leftleftarrows", "amssymb"}, {DATA_DIR "/images/arrows/42.png", "\\leftrightarrows", "amssymb"}, {DATA_DIR "/images/arrows/43.png", "\\rightleftarrows", "amssymb"}, {DATA_DIR "/images/arrows/44.png", "\\rightrightarrows", "amssymb"}, {DATA_DIR "/images/arrows/45.png", "\\downdownarrows", "amssymb"}, {DATA_DIR "/images/arrows/46.png", "\\upuparrows", "amssymb"}, {DATA_DIR "/images/arrows/47.png", "\\circlearrowleft", "amssymb"}, {DATA_DIR "/images/arrows/48.png", "\\circlearrowright", "amssymb"}, {DATA_DIR "/images/arrows/49.png", "\\curvearrowleft", "amssymb"}, {DATA_DIR "/images/arrows/50.png", "\\curvearrowright", "amssymb"}, {DATA_DIR "/images/arrows/51.png", "\\Lsh", "amssymb"}, {DATA_DIR "/images/arrows/52.png", "\\Rsh", "amssymb"}, {DATA_DIR "/images/arrows/53.png", "\\looparrowleft", "amssymb"}, {DATA_DIR "/images/arrows/54.png", "\\looparrowright", "amssymb"}, {DATA_DIR "/images/arrows/55.png", "\\dashleftarrow", "amssymb"}, {DATA_DIR "/images/arrows/56.png", "\\dashrightarrow", "amssymb"}, {DATA_DIR "/images/arrows/57.png", "\\leftrightsquigarrow", "amssymb"}, {DATA_DIR "/images/arrows/58.png", "\\rightsquigarrow", "amssymb"}, {DATA_DIR "/images/arrows/59.png", "\\Lleftarrow", "amssymb"}, {DATA_DIR "/images/arrows/60.png", "\\leftharpoondown", NULL}, {DATA_DIR "/images/arrows/61.png", "\\rightharpoondown", NULL}, {DATA_DIR "/images/arrows/62.png", "\\leftharpoonup", NULL}, {DATA_DIR "/images/arrows/63.png", "\\rightharpoonup", NULL}, {DATA_DIR "/images/arrows/64.png", "\\rightleftharpoons", NULL}, {DATA_DIR "/images/arrows/65.png", "\\leftrightharpoons", "amssymb"}, {DATA_DIR "/images/arrows/66.png", "\\downharpoonleft", "amssymb"}, {DATA_DIR "/images/arrows/67.png", "\\upharpoonleft", "amssymb"}, {DATA_DIR "/images/arrows/68.png", "\\downharpoonright", "amssymb"}, {DATA_DIR "/images/arrows/69.png", "\\upharpoonright", "amssymb"}};
static const SymbolsSymbolInfo SYMBOLS_symbols_relations[139] = {{DATA_DIR "/images/relations/001.png", "\\bowtie", NULL}, {DATA_DIR "/images/relations/002.png", "\\Join", "amssymb"}, {DATA_DIR "/images/relations/003.png", "\\propto", NULL}, {DATA_DIR "/images/relations/004.png", "\\varpropto", "amssymb"}, {DATA_DIR "/images/relations/005.png", "\\multimap", "amssymb"}, {DATA_DIR "/images/relations/006.png", "\\pitchfork", "amssymb"}, {DATA_DIR "/images/relations/007.png", "\\therefore", "amssymb"}, {DATA_DIR "/images/relations/008.png", "\\because", "amssymb"}, {DATA_DIR "/images/relations/009.png", "=", NULL}, {DATA_DIR "/images/relations/010.png", "\\neq", NULL}, {DATA_DIR "/images/relations/011.png", "\\equiv", NULL}, {DATA_DIR "/images/relations/012.png", "\\approx", NULL}, {DATA_DIR "/images/relations/013.png", "\\sim", NULL}, {DATA_DIR "/images/relations/014.png", "\\simeq", NULL}, {DATA_DIR "/images/relations/015.png", "\\backsimeq", "amssymb"}, {DATA_DIR "/images/relations/016.png", "\\approxeq", "amssymb"}, {DATA_DIR "/images/relations/017.png", "\\cong", NULL}, {DATA_DIR "/images/relations/018.png", "\\ncong", "amssymb"}, {DATA_DIR "/images/relations/019.png", "\\smile", NULL}, {DATA_DIR "/images/relations/020.png", "\\frown", NULL}, {DATA_DIR "/images/relations/021.png", "\\asymp", NULL}, {DATA_DIR "/images/relations/022.png", "\\smallfrown", "amssymb"}, {DATA_DIR "/images/relations/023.png", "\\smallsmile", "amssymb"}, {DATA_DIR "/images/relations/024.png", "\\between", "amssymb"}, {DATA_DIR "/images/relations/025.png", "\\prec", NULL}, {DATA_DIR "/images/relations/026.png", "\\succ", NULL}, {DATA_DIR "/images/relations/027.png", "\\nprec", "amssymb"}, {DATA_DIR "/images/relations/028.png", "\\nsucc", "amssymb"}, {DATA_DIR "/images/relations/029.png", "\\preceq", NULL}, {DATA_DIR "/images/relations/030.png", "\\succeq", NULL}, {DATA_DIR "/images/relations/031.png", "\\npreceq", "amssymb"}, {DATA_DIR "/images/relations/032.png", "\\nsucceq", "amssymb"}, {DATA_DIR "/images/relations/033.png", "\\preccurlyeq", "amssymb"}, {DATA_DIR "/images/relations/034.png", "\\succcurlyeq", "amssymb"}, {DATA_DIR "/images/relations/035.png", "\\curlyeqprec", "amssymb"}, {DATA_DIR "/images/relations/036.png", "\\curlyeqsucc", "amssymb"}, {DATA_DIR "/images/relations/037.png", "\\precsim", "amssymb"}, {DATA_DIR "/images/relations/038.png", "\\succsim", "amssymb"}, {DATA_DIR "/images/relations/039.png", "\\precnsim", "amssymb"}, {DATA_DIR "/images/relations/040.png", "\\succnsim", "amssymb"}, {DATA_DIR "/images/relations/041.png", "\\precapprox", "amssymb"}, {DATA_DIR "/images/relations/042.png", "\\succapprox", "amssymb"}, {DATA_DIR "/images/relations/043.png", "\\precnapprox", "amssymb"}, {DATA_DIR "/images/relations/044.png", "\\succnapprox", "amssymb"}, {DATA_DIR "/images/relations/045.png", "\\perp", NULL}, {DATA_DIR "/images/relations/046.png", "\\vdash", NULL}, {DATA_DIR "/images/relations/047.png", "\\dashv", NULL}, {DATA_DIR "/images/relations/048.png", "\\nvdash", "amssymb"}, {DATA_DIR "/images/relations/049.png", "\\Vdash", "amssymb"}, {DATA_DIR "/images/relations/050.png", "\\Vvdash", "amssymb"}, {DATA_DIR "/images/relations/051.png", "\\models", NULL}, {DATA_DIR "/images/relations/052.png", "\\vDash", "amssymb"}, {DATA_DIR "/images/relations/053.png", "\\nvDash", "amssymb"}, {DATA_DIR "/images/relations/054.png", "\\nVDash", "amssymb"}, {DATA_DIR "/images/relations/055.png", "\\mid", NULL}, {DATA_DIR "/images/relations/056.png", "\\nmid", "amssymb"}, {DATA_DIR "/images/relations/057.png", "\\parallel", NULL}, {DATA_DIR "/images/relations/058.png", "\\nparallel", "amssymb"}, {DATA_DIR "/images/relations/059.png", "\\shortmid", "amssymb"}, {DATA_DIR "/images/relations/060.png", "\\nshortmid", "amssymb"}, {DATA_DIR "/images/relations/061.png", "\\shortparallel", "amssymb"}, {DATA_DIR "/images/relations/062.png", "\\nshortparallel", "amssymb"}, {DATA_DIR "/images/relations/063.png", "<", NULL}, {DATA_DIR "/images/relations/064.png", ">", NULL}, {DATA_DIR "/images/relations/065.png", "\\nless", "amssymb"}, {DATA_DIR "/images/relations/066.png", "\\ngtr", "amssymb"}, {DATA_DIR "/images/relations/067.png", "\\lessdot", "amssymb"}, {DATA_DIR "/images/relations/068.png", "\\gtrdot", "amssymb"}, {DATA_DIR "/images/relations/069.png", "\\ll", NULL}, {DATA_DIR "/images/relations/070.png", "\\gg", NULL}, {DATA_DIR "/images/relations/071.png", "\\lll", "amssymb"}, {DATA_DIR "/images/relations/072.png", "\\ggg", "amssymb"}, {DATA_DIR "/images/relations/073.png", "\\leq", NULL}, {DATA_DIR "/images/relations/074.png", "\\geq", NULL}, {DATA_DIR "/images/relations/075.png", "\\lneq", "amssymb"}, {DATA_DIR "/images/relations/076.png", "\\gneq", "amssymb"}, {DATA_DIR "/images/relations/077.png", "\\nleq", "amssymb"}, {DATA_DIR "/images/relations/078.png", "\\ngeq", "amssymb"}, {DATA_DIR "/images/relations/079.png", "\\leqq", "amssymb"}, {DATA_DIR "/images/relations/080.png", "\\geqq", "amssymb"}, {DATA_DIR "/images/relations/081.png", "\\lneqq", "amssymb"}, {DATA_DIR "/images/relations/082.png", "\\gneqq", "amssymb"}, {DATA_DIR "/images/relations/083.png", "\\lvertneqq", "amssymb"}, {DATA_DIR "/images/relations/084.png", "\\gvertneqq", "amssymb"}, {DATA_DIR "/images/relations/085.png", "\\nleqq", "amssymb"}, {DATA_DIR "/images/relations/086.png", "\\ngeqq", "amssymb"}, {DATA_DIR "/images/relations/087.png", "\\leqslant", "amssymb"}, {DATA_DIR "/images/relations/088.png", "\\geqslant", "amssymb"}, {DATA_DIR "/images/relations/089.png", "\\nleqslant", "amssymb"}, {DATA_DIR "/images/relations/090.png", "\\ngeqslant", "amssymb"}, {DATA_DIR "/images/relations/091.png", "\\eqslantless", "amssymb"}, {DATA_DIR "/images/relations/092.png", "\\eqslantgtr", "amssymb"}, {DATA_DIR "/images/relations/093.png", "\\lessgtr", "amssymb"}, {DATA_DIR "/images/relations/094.png", "\\gtrless", "amssymb"}, {DATA_DIR "/images/relations/095.png", "\\lesseqgtr", "amssymb"}, {DATA_DIR "/images/relations/096.png", "\\gtreqless", "amssymb"}, {DATA_DIR "/images/relations/097.png", "\\lesseqqgtr", "amssymb"}, {DATA_DIR "/images/relations/098.png", "\\gtreqqless", "amssymb"}, {DATA_DIR "/images/relations/099.png", "\\lesssim", "amssymb"}, {DATA_DIR "/images/relations/100.png", "\\gtrsim", "amssymb"}, {DATA_DIR "/images/relations/101.png", "\\lnsim", "amssymb"}, {DATA_DIR "/images/relations/102.png", "\\gnsim", "amssymb"}, {DATA_DIR "/images/relations/103.png", "\\lessapprox", "amssymb"}, {DATA_DIR "/images/relations/104.png", "\\gtrapprox", "amssymb"}, {DATA_DIR "/images/relations/105.png", "\\lnapprox", "amssymb"}, {DATA_DIR "/images/relations/106.png", "\\gnapprox", "amssymb"}, {DATA_DIR "/images/relations/107.png", "\\vartriangleleft", "amssymb"}, {DATA_DIR "/images/relations/108.png", "\\vartriangleright", "amssymb"}, {DATA_DIR "/images/relations/109.png", "\\ntriangleleft", "amssymb"}, {DATA_DIR "/images/relations/110.png", "\\ntriangleright", "amssymb"}, {DATA_DIR "/images/relations/111.png", "\\trianglelefteq", "amssymb"}, {DATA_DIR "/images/relations/112.png", "\\trianglerighteq", "amssymb"}, {DATA_DIR "/images/relations/113.png", "\\ntrianglelefteq", "amssymb"}, {DATA_DIR "/images/relations/114.png", "\\ntrianglerighteq", "amssymb"}, {DATA_DIR "/images/relations/115.png", "\\blacktriangleleft", "amssymb"}, {DATA_DIR "/images/relations/116.png", "\\blacktriangleright", "amssymb"}, {DATA_DIR "/images/relations/117.png", "\\subset", NULL}, {DATA_DIR "/images/relations/118.png", "\\supset", NULL}, {DATA_DIR "/images/relations/119.png", "\\subseteq", NULL}, {DATA_DIR "/images/relations/120.png", "\\supseteq", NULL}, {DATA_DIR "/images/relations/121.png", "\\subsetneq", "amssymb"}, {DATA_DIR "/images/relations/122.png", "\\supsetneq", "amssymb"}, {DATA_DIR "/images/relations/123.png", "\\varsubsetneq", "amssymb"}, {DATA_DIR "/images/relations/124.png", "\\varsupsetneq", "amssymb"}, {DATA_DIR "/images/relations/125.png", "\\nsubseteq", "amssymb"}, {DATA_DIR "/images/relations/126.png", "\\nsupseteq", "amssymb"}, {DATA_DIR "/images/relations/127.png", "\\subseteqq", "amssymb"}, {DATA_DIR "/images/relations/128.png", "\\supseteqq", "amssymb"}, {DATA_DIR "/images/relations/129.png", "\\subsetneqq", "amssymb"}, {DATA_DIR "/images/relations/130.png", "\\supsetneqq", "amssymb"}, {DATA_DIR "/images/relations/131.png", "\\nsubseteqq", "amssymb"}, {DATA_DIR "/images/relations/132.png", "\\nsupseteqq", "amssymb"}, {DATA_DIR "/images/relations/133.png", "\\backepsilon", "amssymb"}, {DATA_DIR "/images/relations/134.png", "\\Subset", "amssymb"}, {DATA_DIR "/images/relations/135.png", "\\Supset", "amssymb"}, {DATA_DIR "/images/relations/136.png", "\\sqsubset", "amssymb"}, {DATA_DIR "/images/relations/137.png", "\\sqsupset", "amssymb"}, {DATA_DIR "/images/relations/138.png", "\\sqsubseteq", NULL}, {DATA_DIR "/images/relations/139.png", "\\sqsupseteq", NULL}};
static const SymbolsSymbolInfo SYMBOLS_symbols_operators[80] = {{DATA_DIR "/images/operators/001.png", "\\pm", NULL}, {DATA_DIR "/images/operators/002.png", "\\mp", NULL}, {DATA_DIR "/images/operators/003.png", "\\times", NULL}, {DATA_DIR "/images/operators/004.png", "\\div", NULL}, {DATA_DIR "/images/operators/005.png", "\\ast", NULL}, {DATA_DIR "/images/operators/006.png", "\\star", NULL}, {DATA_DIR "/images/operators/007.png", "\\circ", NULL}, {DATA_DIR "/images/operators/008.png", "\\bullet", NULL}, {DATA_DIR "/images/operators/009.png", "\\divideontimes", "amssymb"}, {DATA_DIR "/images/operators/010.png", "\\ltimes", "amssymb"}, {DATA_DIR "/images/operators/011.png", "\\rtimes", "amssymb"}, {DATA_DIR "/images/operators/012.png", "\\cdot", NULL}, {DATA_DIR "/images/operators/013.png", "\\dotplus", "amssymb"}, {DATA_DIR "/images/operators/014.png", "\\leftthreetimes", "amssymb"}, {DATA_DIR "/images/operators/015.png", "\\rightthreetimes", "amssymb"}, {DATA_DIR "/images/operators/016.png", "\\amalg", NULL}, {DATA_DIR "/images/operators/017.png", "\\otimes", NULL}, {DATA_DIR "/images/operators/018.png", "\\oplus", NULL}, {DATA_DIR "/images/operators/019.png", "\\ominus", NULL}, {DATA_DIR "/images/operators/020.png", "\\oslash", NULL}, {DATA_DIR "/images/operators/021.png", "\\odot", NULL}, {DATA_DIR "/images/operators/022.png", "\\circledcirc", "amssymb"}, {DATA_DIR "/images/operators/023.png", "\\circleddash", "amssymb"}, {DATA_DIR "/images/operators/024.png", "\\circledast", "amssymb"}, {DATA_DIR "/images/operators/025.png", "\\bigcirc", NULL}, {DATA_DIR "/images/operators/026.png", "\\boxdot", "amssymb"}, {DATA_DIR "/images/operators/027.png", "\\boxminus", "amssymb"}, {DATA_DIR "/images/operators/028.png", "\\boxplus", "amssymb"}, {DATA_DIR "/images/operators/029.png", "\\boxtimes", "amssymb"}, {DATA_DIR "/images/operators/030.png", "\\diamond", NULL}, {DATA_DIR "/images/operators/031.png", "\\bigtriangleup", NULL}, {DATA_DIR "/images/operators/032.png", "\\bigtriangledown", NULL}, {DATA_DIR "/images/operators/033.png", "\\triangleleft", NULL}, {DATA_DIR "/images/operators/034.png", "\\triangleright", NULL}, {DATA_DIR "/images/operators/035.png", "\\lhd", "amssymb"}, {DATA_DIR "/images/operators/036.png", "\\rhd", "amssymb"}, {DATA_DIR "/images/operators/037.png", "\\unlhd", "amssymb"}, {DATA_DIR "/images/operators/038.png", "\\unrhd", "amssymb"}, {DATA_DIR "/images/operators/039.png", "\\cup", NULL}, {DATA_DIR "/images/operators/040.png", "\\cap", NULL}, {DATA_DIR "/images/operators/041.png", "\\uplus", NULL}, {DATA_DIR "/images/operators/042.png", "\\Cup", "amssymb"}, {DATA_DIR "/images/operators/043.png", "\\Cap", "amssymb"}, {DATA_DIR "/images/operators/044.png", "\\wr", NULL}, {DATA_DIR "/images/operators/045.png", "\\setminus", NULL}, {DATA_DIR "/images/operators/046.png", "\\smallsetminus", "amssymb"}, {DATA_DIR "/images/operators/047.png", "\\sqcap", NULL}, {DATA_DIR "/images/operators/048.png", "\\sqcup", NULL}, {DATA_DIR "/images/operators/049.png", "\\wedge", NULL}, {DATA_DIR "/images/operators/050.png", "\\vee", NULL}, {DATA_DIR "/images/operators/051.png", "\\barwedge", "amssymb"}, {DATA_DIR "/images/operators/052.png", "\\veebar", "amssymb"}, {DATA_DIR "/images/operators/053.png", "\\doublebarwedge", "amssymb"}, {DATA_DIR "/images/operators/054.png", "\\curlywedge", "amssymb"}, {DATA_DIR "/images/operators/055.png", "\\curlyvee", "amssymb"}, {DATA_DIR "/images/operators/056.png", "\\dagger", "amssymb"}, {DATA_DIR "/images/operators/057.png", "\\ddagger", "amssymb"}, {DATA_DIR "/images/operators/058.png", "\\intercal", "amssymb"}, {DATA_DIR "/images/operators/059.png", "\\bigcap", NULL}, {DATA_DIR "/images/operators/060.png", "\\bigcup", NULL}, {DATA_DIR "/images/operators/061.png", "\\biguplus", NULL}, {DATA_DIR "/images/operators/062.png", "\\bigsqcup", NULL}, {DATA_DIR "/images/operators/063.png", "\\prod", NULL}, {DATA_DIR "/images/operators/064.png", "\\coprod", NULL}, {DATA_DIR "/images/operators/065.png", "\\bigwedge", NULL}, {DATA_DIR "/images/operators/066.png", "\\bigvee", NULL}, {DATA_DIR "/images/operators/067.png", "\\bigodot", NULL}, {DATA_DIR "/images/operators/068.png", "\\bigoplus", NULL}, {DATA_DIR "/images/operators/069.png", "\\bigotimes", NULL}, {DATA_DIR "/images/operators/070.png", "\\sum", NULL}, {DATA_DIR "/images/operators/071.png", "\\int", NULL}, {DATA_DIR "/images/operators/072.png", "\\oint", NULL}, {DATA_DIR "/images/operators/073.png", "\\iint", "amsmath"}, {DATA_DIR "/images/operators/074.png", "\\iiint", "amsmath"}, {DATA_DIR "/images/operators/075.png", "\\iiiint", "amsmath"}, {DATA_DIR "/images/operators/076.png", "\\idotsint", "amsmath"}, {DATA_DIR "/images/operators/077.png", "\\varlimsup", "amsmath"}, {DATA_DIR "/images/operators/078.png", "\\varliminf", "amsmath"}, {DATA_DIR "/images/operators/079.png", "\\varinjlim", "amsmath"}, {DATA_DIR "/images/operators/080.png", "\\varprojlim", "amsmath"}};
static const SymbolsSymbolInfo SYMBOLS_symbols_delimiters[37] = {{DATA_DIR "/images/delimiters/01.png", "\\downarrow", NULL}, {DATA_DIR "/images/delimiters/02.png", "\\Downarrow", NULL}, {DATA_DIR "/images/delimiters/03.png", "[", NULL}, {DATA_DIR "/images/delimiters/04.png", "]", NULL}, {DATA_DIR "/images/delimiters/05.png", "\\langle", NULL}, {DATA_DIR "/images/delimiters/06.png", "\\rangle", NULL}, {DATA_DIR "/images/delimiters/07.png", "|", NULL}, {DATA_DIR "/images/delimiters/08.png", "\\|", NULL}, {DATA_DIR "/images/delimiters/09.png", "\\lceil", NULL}, {DATA_DIR "/images/delimiters/10.png", "\\rceil", NULL}, {DATA_DIR "/images/delimiters/11.png", "\\uparrow", NULL}, {DATA_DIR "/images/delimiters/12.png", "\\Uparrow", NULL}, {DATA_DIR "/images/delimiters/13.png", "\\lfloor", NULL}, {DATA_DIR "/images/delimiters/14.png", "\\rfloor", NULL}, {DATA_DIR "/images/delimiters/15.png", "\\updownarrow", NULL}, {DATA_DIR "/images/delimiters/16.png", "\\Updownarrow", NULL}, {DATA_DIR "/images/delimiters/17.png", "(", NULL}, {DATA_DIR "/images/delimiters/18.png", ")", NULL}, {DATA_DIR "/images/delimiters/19.png", "\\{", NULL}, {DATA_DIR "/images/delimiters/20.png", "\\}", NULL}, {DATA_DIR "/images/delimiters/21.png", "/", NULL}, {DATA_DIR "/images/delimiters/22.png", "\\backslash", NULL}, {DATA_DIR "/images/delimiters/23.png", "\\lmoustache", NULL}, {DATA_DIR "/images/delimiters/24.png", "\\rmoustache", NULL}, {DATA_DIR "/images/delimiters/25.png", "\\lgroup", NULL}, {DATA_DIR "/images/delimiters/26.png", "\\rgroup", NULL}, {DATA_DIR "/images/delimiters/27.png", "\\arrowvert", NULL}, {DATA_DIR "/images/delimiters/28.png", "\\Arrowvert", NULL}, {DATA_DIR "/images/delimiters/29.png", "\\bracevert", NULL}, {DATA_DIR "/images/delimiters/30.png", "\\lvert", "amsmath"}, {DATA_DIR "/images/delimiters/31.png", "\\rvert", "amsmath"}, {DATA_DIR "/images/delimiters/32.png", "\\lVert", "amsmath"}, {DATA_DIR "/images/delimiters/33.png", "\\rVert", "amsmath"}, {DATA_DIR "/images/delimiters/34.png", "\\ulcorner", "amssymb"}, {DATA_DIR "/images/delimiters/35.png", "\\urcorner", "amssymb"}, {DATA_DIR "/images/delimiters/36.png", "\\llcorner", "amssymb"}, {DATA_DIR "/images/delimiters/37.png", "\\lrcorner", "amssymb"}};
static const SymbolsSymbolInfo SYMBOLS_symbols_misc_math[103] = {{DATA_DIR "/images/misc-math/01.png", "\\cdotp", NULL}, {DATA_DIR "/images/misc-math/02.png", "\\colon", NULL}, {DATA_DIR "/images/misc-math/03.png", "\\ldotp", NULL}, {DATA_DIR "/images/misc-math/04.png", "\\vdots", NULL}, {DATA_DIR "/images/misc-math/05.png", "\\cdots", NULL}, {DATA_DIR "/images/misc-math/06.png", "\\ddots", NULL}, {DATA_DIR "/images/misc-math/07.png", "\\ldots", NULL}, {DATA_DIR "/images/misc-math/08.png", "\\neg", NULL}, {DATA_DIR "/images/misc-math/09.png", "\\infty", NULL}, {DATA_DIR "/images/misc-math/10.png", "\\prime", NULL}, {DATA_DIR "/images/misc-math/11.png", "\\backprime", "amssymb"}, {DATA_DIR "/images/misc-math/12.png", "\\backslash", NULL}, {DATA_DIR "/images/misc-math/13.png", "\\diagdown", "amssymb"}, {DATA_DIR "/images/misc-math/14.png", "\\diagup", "amssymb"}, {DATA_DIR "/images/misc-math/15.png", "\\surd", NULL}, {DATA_DIR "/images/misc-math/16.png", "\\emptyset", NULL}, {DATA_DIR "/images/misc-math/17.png", "\\varnothing", "amssymb"}, {DATA_DIR "/images/misc-math/18.png", "\\sharp", NULL}, {DATA_DIR "/images/misc-math/19.png", "\\flat", NULL}, {DATA_DIR "/images/misc-math/20.png", "\\natural", NULL}, {DATA_DIR "/images/misc-math/21.png", "\\angle", NULL}, {DATA_DIR "/images/misc-math/22.png", "\\sphericalangle", "amssymb"}, {DATA_DIR "/images/misc-math/23.png", "\\measuredangle", "amssymb"}, {DATA_DIR "/images/misc-math/24.png", "\\Box", "amssymb"}, {DATA_DIR "/images/misc-math/25.png", "\\square", "amssymb"}, {DATA_DIR "/images/misc-math/26.png", "\\triangle", NULL}, {DATA_DIR "/images/misc-math/27.png", "\\vartriangle", "amssymb"}, {DATA_DIR "/images/misc-math/28.png", "\\triangledown", "amssymb"}, {DATA_DIR "/images/misc-math/29.png", "\\Diamond", "amssymb"}, {DATA_DIR "/images/misc-math/30.png", "\\diamondsuit", NULL}, {DATA_DIR "/images/misc-math/31.png", "\\lozenge", "amssymb"}, {DATA_DIR "/images/misc-math/32.png", "\\heartsuit", NULL}, {DATA_DIR "/images/misc-math/33.png", "\\blacksquare", "amssymb"}, {DATA_DIR "/images/misc-math/34.png", "\\blacktriangle", "amssymb"}, {DATA_DIR "/images/misc-math/35.png", "\\blacktriangledown", "amssymb"}, {DATA_DIR "/images/misc-math/36.png", "\\blacklozenge", "amssymb"}, {DATA_DIR "/images/misc-math/37.png", "\\bigstar", "amssymb"}, {DATA_DIR "/images/misc-math/38.png", "\\spadesuit", NULL}, {DATA_DIR "/images/misc-math/39.png", "\\clubsuit", NULL}, {DATA_DIR "/images/misc-math/40.png", "\\forall", NULL}, {DATA_DIR "/images/misc-math/41.png", "\\exists", NULL}, {DATA_DIR "/images/misc-math/42.png", "\\nexists", "amssymb"}, {DATA_DIR "/images/misc-math/43.png", "\\Finv", "amssymb"}, {DATA_DIR "/images/misc-math/44.png", "\\Game", "amssymb"}, {DATA_DIR "/images/misc-math/45.png", "\\ni", NULL}, {DATA_DIR "/images/misc-math/46.png", "\\in", NULL}, {DATA_DIR "/images/misc-math/47.png", "\\notin", NULL}, {DATA_DIR "/images/misc-math/48.png", "\\complement", "amssymb"}, {DATA_DIR "/images/misc-math/set-N.png", "\\mathbb{N}", "amsfonts"}, {DATA_DIR "/images/misc-math/set-Z.png", "\\mathbb{Z}", "amsfonts"}, {DATA_DIR "/images/misc-math/set-Q.png", "\\mathbb{Q}", "amsfonts"}, {DATA_DIR "/images/misc-math/set-I.png", "\\mathbb{I}", "amsfonts"}, {DATA_DIR "/images/misc-math/set-R.png", "\\mathbb{R}", "amsfonts"}, {DATA_DIR "/images/misc-math/set-C.png", "\\mathbb{C}", "amsfonts"}, {DATA_DIR "/images/misc-math/49.png", "\\Im", NULL}, {DATA_DIR "/images/misc-math/50.png", "\\Re", NULL}, {DATA_DIR "/images/misc-math/51.png", "\\aleph", NULL}, {DATA_DIR "/images/misc-math/52.png", "\\wp", NULL}, {DATA_DIR "/images/misc-math/53.png", "\\hslash", "amssymb"}, {DATA_DIR "/images/misc-math/54.png", "\\hbar", NULL}, {DATA_DIR "/images/misc-math/55.png", "\\imath", NULL}, {DATA_DIR "/images/misc-math/56.png", "\\jmath", NULL}, {DATA_DIR "/images/misc-math/57.png", "\\Bbbk", "amssymb"}, {DATA_DIR "/images/misc-math/58.png", "\\ell", NULL}, {DATA_DIR "/images/misc-math/59.png", "\\circledR", "amssymb"}, {DATA_DIR "/images/misc-math/60.png", "\\circledS", "amssymb"}, {DATA_DIR "/images/misc-math/61.png", "\\bot", NULL}, {DATA_DIR "/images/misc-math/62.png", "\\top", NULL}, {DATA_DIR "/images/misc-math/63.png", "\\partial", NULL}, {DATA_DIR "/images/misc-math/64.png", "\\nabla", NULL}, {DATA_DIR "/images/misc-math/65.png", "\\eth", "amssymb"}, {DATA_DIR "/images/misc-math/66.png", "\\mho", "amssymb"}, {DATA_DIR "/images/misc-math/67.png", "\\acute{}", NULL}, {DATA_DIR "/images/misc-math/68.png", "\\grave{}", NULL}, {DATA_DIR "/images/misc-math/69.png", "\\check{}", NULL}, {DATA_DIR "/images/misc-math/70.png", "\\hat{}", NULL}, {DATA_DIR "/images/misc-math/71.png", "\\tilde{}", NULL}, {DATA_DIR "/images/misc-math/72.png", "\\bar{}", NULL}, {DATA_DIR "/images/misc-math/73.png", "\\vec{}", NULL}, {DATA_DIR "/images/misc-math/74.png", "\\breve{}", NULL}, {DATA_DIR "/images/misc-math/75.png", "\\dot{}", NULL}, {DATA_DIR "/images/misc-math/76.png", "\\ddot{}", NULL}, {DATA_DIR "/images/misc-math/77.png", "\\dddot{}", "amsmath"}, {DATA_DIR "/images/misc-math/78.png", "\\ddddot{}", "amsmath"}, {DATA_DIR "/images/misc-math/79.png", "\\mathring{}", NULL}, {DATA_DIR "/images/misc-math/80.png", "\\widetilde{}", NULL}, {DATA_DIR "/images/misc-math/81.png", "\\widehat{}", NULL}, {DATA_DIR "/images/misc-math/82.png", "\\overleftarrow{}", NULL}, {DATA_DIR "/images/misc-math/83.png", "\\overrightarrow{}", NULL}, {DATA_DIR "/images/misc-math/84.png", "\\overline{}", NULL}, {DATA_DIR "/images/misc-math/85.png", "\\underline{}", NULL}, {DATA_DIR "/images/misc-math/86.png", "\\overbrace{}", NULL}, {DATA_DIR "/images/misc-math/87.png", "\\underbrace{}", NULL}, {DATA_DIR "/images/misc-math/88.png", "\\overleftrightarrow{}", "amsmath"}, {DATA_DIR "/images/misc-math/89.png", "\\underleftrightarrow{}", "amsmath"}, {DATA_DIR "/images/misc-math/90.png", "\\underleftarrow{}", "amsmath"}, {DATA_DIR "/images/misc-math/91.png", "\\underrightarrow{}", "amsmath"}, {DATA_DIR "/images/misc-math/92.png", "\\xleftarrow{}", "amsmath"}, {DATA_DIR "/images/misc-math/93.png", "\\xrightarrow{}", "amsmath"}, {DATA_DIR "/images/misc-math/94.png", "\\stackrel{}{}", NULL}, {DATA_DIR "/images/misc-math/95.png", "\\sqrt{}", NULL}, {DATA_DIR "/images/misc-math/96.png", "f'", NULL}, {DATA_DIR "/images/misc-math/97.png", "f''", NULL}};
static const SymbolsSymbolInfo SYMBOLS_symbols_misc_text[135] = {{DATA_DIR "/images/misc-text/001.png", "\\dots", NULL}, {DATA_DIR "/images/misc-text/002.png", "\\texttildelow", "textcomp"}, {DATA_DIR "/images/misc-text/003.png", "\\textasciicircum", NULL}, {DATA_DIR "/images/misc-text/004.png", "\\textasciimacron", "textcomp"}, {DATA_DIR "/images/misc-text/005.png", "\\textasciiacute", "textcomp"}, {DATA_DIR "/images/misc-text/006.png", "\\textasciidieresis", "textcomp"}, {DATA_DIR "/images/misc-text/007.png", "\\textasciitilde", NULL}, {DATA_DIR "/images/misc-text/008.png", "\\textasciigrave", "textcomp"}, {DATA_DIR "/images/misc-text/009.png", "\\textasciibreve", "textcomp"}, {DATA_DIR "/images/misc-text/010.png", "\\textasciicaron", "textcomp"}, {DATA_DIR "/images/misc-text/011.png", "\\textacutedbl", "textcomp"}, {DATA_DIR "/images/misc-text/012.png", "\\textgravedbl", "textcomp"}, {DATA_DIR "/images/misc-text/013.png", "\\textquotedblleft", NULL}, {DATA_DIR "/images/misc-text/014.png", "\\textquotedblright", NULL}, {DATA_DIR "/images/misc-text/015.png", "\\textquoteleft", NULL}, {DATA_DIR "/images/misc-text/016.png", "\\textquoteright", NULL}, {DATA_DIR "/images/misc-text/017.png", "\\textquotestraightbase", "textcomp"}, {DATA_DIR "/images/misc-text/018.png", "\\textquotestraightdblbase", "textcomp"}, {DATA_DIR "/images/misc-text/019.png", "\\textquotesingle", "textcomp"}, {DATA_DIR "/images/misc-text/020.png", "\\textdblhyphen", "textcomp"}, {DATA_DIR "/images/misc-text/021.png", "\\textdblhyphenchar", "textcomp"}, {DATA_DIR "/images/misc-text/022.png", "\\textasteriskcentered", NULL}, {DATA_DIR "/images/misc-text/023.png", "\\textperiodcentered", NULL}, {DATA_DIR "/images/misc-text/024.png", "\\textquestiondown", NULL}, {DATA_DIR "/images/misc-text/025.png", "\\textinterrobang", "textcomp"}, {DATA_DIR "/images/misc-text/026.png", "\\textinterrobangdown", "textcomp"}, {DATA_DIR "/images/misc-text/027.png", "\\textexclamdown", NULL}, {DATA_DIR "/images/misc-text/028.png", "\\texttwelveudash", "textcomp"}, {DATA_DIR "/images/misc-text/029.png", "\\textemdash", NULL}, {DATA_DIR "/images/misc-text/030.png", "\\textendash", NULL}, {DATA_DIR "/images/misc-text/031.png", "\\textthreequartersemdash", "textcomp"}, {DATA_DIR "/images/misc-text/032.png", "\\textvisiblespace", NULL}, {DATA_DIR "/images/misc-text/033.png", "\\_", NULL}, {DATA_DIR "/images/misc-text/034.png", "\\textcurrency", "textcomp"}, {DATA_DIR "/images/misc-text/035.png", "\\textbaht", "textcomp"}, {DATA_DIR "/images/misc-text/036.png", "\\textguarani", "textcomp"}, {DATA_DIR "/images/misc-text/037.png", "\\textwon", "textcomp"}, {DATA_DIR "/images/misc-text/038.png", "\\textcent", "textcomp"}, {DATA_DIR "/images/misc-text/039.png", "\\textcentoldstyle", "textcomp"}, {DATA_DIR "/images/misc-text/040.png", "\\textdollar", NULL}, {DATA_DIR "/images/misc-text/041.png", "\\textdollaroldstyle", "textcomp"}, {DATA_DIR "/images/misc-text/042.png", "\\textlira", "textcomp"}, {DATA_DIR "/images/misc-text/043.png", "\\textyen", "textcomp"}, {DATA_DIR "/images/misc-text/044.png", "\\textdong", "textcomp"}, {DATA_DIR "/images/misc-text/045.png", "\\textnaira", "textcomp"}, {DATA_DIR "/images/misc-text/046.png", "\\textcolonmonetary", "textcomp"}, {DATA_DIR "/images/misc-text/047.png", "\\textpeso", "textcomp"}, {DATA_DIR "/images/misc-text/048.png", "\\pounds", NULL}, {DATA_DIR "/images/misc-text/049.png", "\\textflorin", "textcomp"}, {DATA_DIR "/images/misc-text/050.png", "\\texteuro", "textcomp"}, {DATA_DIR "/images/misc-text/051.png", "\\geneuro", "eurosym"}, {DATA_DIR "/images/misc-text/052.png", "\\geneuronarrow", "eurosym"}, {DATA_DIR "/images/misc-text/053.png", "\\geneurowide", "eurosym"}, {DATA_DIR "/images/misc-text/054.png", "\\officialeuro", "eurosym"}, {DATA_DIR "/images/misc-text/055.png", "\\textcircled{a}", NULL}, {DATA_DIR "/images/misc-text/056.png", "\\textcopyright", NULL}, {DATA_DIR "/images/misc-text/057.png", "\\textcopyleft", "textcomp"}, {DATA_DIR "/images/misc-text/058.png", "\\textregistered", NULL}, {DATA_DIR "/images/misc-text/059.png", "\\texttrademark", NULL}, {DATA_DIR "/images/misc-text/060.png", "\\textservicemark", "textcomp"}, {DATA_DIR "/images/misc-text/061.png", "\\oldstylenums{0}", NULL}, {DATA_DIR "/images/misc-text/062.png", "\\oldstylenums{1}", NULL}, {DATA_DIR "/images/misc-text/063.png", "\\oldstylenums{2}", NULL}, {DATA_DIR "/images/misc-text/064.png", "\\oldstylenums{3}", NULL}, {DATA_DIR "/images/misc-text/065.png", "\\oldstylenums{4}", NULL}, {DATA_DIR "/images/misc-text/066.png", "\\oldstylenums{5}", NULL}, {DATA_DIR "/images/misc-text/067.png", "\\oldstylenums{6}", NULL}, {DATA_DIR "/images/misc-text/068.png", "\\oldstylenums{7}", NULL}, {DATA_DIR "/images/misc-text/069.png", "\\oldstylenums{8}", NULL}, {DATA_DIR "/images/misc-text/070.png", "\\oldstylenums{9}", NULL}, {DATA_DIR "/images/misc-text/071.png", "\\textonehalf", "textcomp"}, {DATA_DIR "/images/misc-text/072.png", "\\textonequarter", "textcomp"}, {DATA_DIR "/images/misc-text/073.png", "\\textthreequarters", "textcomp"}, {DATA_DIR "/images/misc-text/074.png", "\\textonesuperior", "textcomp"}, {DATA_DIR "/images/misc-text/075.png", "\\texttwosuperior", "textcomp"}, {DATA_DIR "/images/misc-text/076.png", "\\textthreesuperior", "textcomp"}, {DATA_DIR "/images/misc-text/077.png", "\\textnumero", "textcomp"}, {DATA_DIR "/images/misc-text/078.png", "\\textpertenthousand", "textcomp"}, {DATA_DIR "/images/misc-text/079.png", "\\textperthousand", "textcomp"}, {DATA_DIR "/images/misc-text/080.png", "\\textdiscount", "textcomp"}, {DATA_DIR "/images/misc-text/081.png", "\\textblank", "textcomp"}, {DATA_DIR "/images/misc-text/082.png", "\\textrecipe", "textcomp"}, {DATA_DIR "/images/misc-text/083.png", "\\textestimated", "textcomp"}, {DATA_DIR "/images/misc-text/084.png", "\\textreferencemark", "textcomp"}, {DATA_DIR "/images/misc-text/085.png", "\\textmusicalnote", "textcomp"}, {DATA_DIR "/images/misc-text/086.png", "\\dag", NULL}, {DATA_DIR "/images/misc-text/087.png", "\\ddag", NULL}, {DATA_DIR "/images/misc-text/088.png", "\\S", NULL}, {DATA_DIR "/images/misc-text/089.png", "\\$", NULL}, {DATA_DIR "/images/misc-text/090.png", "\\textpilcrow", "textcomp"}, {DATA_DIR "/images/misc-text/091.png", "\\Cutleft", "marvosym"}, {DATA_DIR "/images/misc-text/092.png", "\\Cutright", "marvosym"}, {DATA_DIR "/images/misc-text/093.png", "\\Leftscissors", "marvosym"}, {DATA_DIR "/images/misc-text/094.png", "\\Cutline", "marvosym"}, {DATA_DIR "/images/misc-text/095.png", "\\Kutline", "marvosym"}, {DATA_DIR "/images/misc-text/096.png", "\\Rightscissors", "marvosym"}, {DATA_DIR "/images/misc-text/097.png", "\\CheckedBox", "wasysym"}, {DATA_DIR "/images/misc-text/098.png", "\\Square", "wasysym"}, {DATA_DIR "/images/misc-text/099.png", "\\XBox", "wasysym"}, {DATA_DIR "/images/misc-text/100.png", "\\textbigcircle", "textcomp"}, {DATA_DIR "/images/misc-text/101.png", "\\textopenbullet", "textcomp"}, {DATA_DIR "/images/misc-text/102.png", "\\textbullet", NULL}, {DATA_DIR "/images/misc-text/103.png", "\\checkmark", "amssymb"}, {DATA_DIR "/images/misc-text/104.png", "\\maltese", "amssymb"}, {DATA_DIR "/images/misc-text/105.png", "\\textordmasculine", "textcomp"}, {DATA_DIR "/images/misc-text/106.png", "\\textordfeminine", "textcomp"}, {DATA_DIR "/images/misc-text/107.png", "\\textborn", "textcomp"}, {DATA_DIR "/images/misc-text/108.png", "\\textdivorced", "textcomp"}, {DATA_DIR "/images/misc-text/109.png", "\\textdied", "textcomp"}, {DATA_DIR "/images/misc-text/110.png", "\\textmarried", "textcomp"}, {DATA_DIR "/images/misc-text/111.png", "\\textleaf", "textcomp"}, {DATA_DIR "/images/misc-text/112.png", "\\textcelsius", "textcomp"}, {DATA_DIR "/images/misc-text/113.png", "\\textdegree", "textcomp"}, {DATA_DIR "/images/misc-text/114.png", "\\textmho", "textcomp"}, {DATA_DIR "/images/misc-text/115.png", "\\textohm", "textcomp"}, {DATA_DIR "/images/misc-text/116.png", "\\textbackslash", NULL}, {DATA_DIR "/images/misc-text/117.png", "\\textbar", NULL}, {DATA_DIR "/images/misc-text/118.png", "\\textbrokenbar", "textcomp"}, {DATA_DIR "/images/misc-text/119.png", "\\textbardbl", NULL}, {DATA_DIR "/images/misc-text/120.png", "\\textfractionsolidus", "textcomp"}, {DATA_DIR "/images/misc-text/121.png", "\\textlangle", "textcomp"}, {DATA_DIR "/images/misc-text/122.png", "\\textlnot", "textcomp"}, {DATA_DIR "/images/misc-text/123.png", "\\textminus", "textcomp"}, {DATA_DIR "/images/misc-text/124.png", "\\textrangle", "textcomp"}, {DATA_DIR "/images/misc-text/125.png", "\\textlbrackdbl", "textcomp"}, {DATA_DIR "/images/misc-text/126.png", "\\textrbrackdbl", "textcomp"}, {DATA_DIR "/images/misc-text/127.png", "\\textmu", "textcomp"}, {DATA_DIR "/images/misc-text/128.png", "\\textpm", "textcomp"}, {DATA_DIR "/images/misc-text/129.png", "\\textlquill", "textcomp"}, {DATA_DIR "/images/misc-text/130.png", "\\textrquill", "textcomp"}, {DATA_DIR "/images/misc-text/131.png", "\\textless", NULL}, {DATA_DIR "/images/misc-text/132.png", "\\textgreater", NULL}, {DATA_DIR "/images/misc-text/133.png", "\\textsurd", "textcomp"}, {DATA_DIR "/images/misc-text/134.png", "\\texttimes", "textcomp"}, {DATA_DIR "/images/misc-text/135.png", "\\textdiv", "textcomp"}};


static GType symbols_symbol_column_get_type (void) {
	static volatile gsize symbols_symbol_column_type_id__volatile = 0;
	if (g_once_init_enter (&symbols_symbol_column_type_id__volatile)) {
		static const GEnumValue values[] = {{SYMBOLS_SYMBOL_COLUMN_PIXBUF, "SYMBOLS_SYMBOL_COLUMN_PIXBUF", "pixbuf"}, {SYMBOLS_SYMBOL_COLUMN_COMMAND, "SYMBOLS_SYMBOL_COLUMN_COMMAND", "command"}, {SYMBOLS_SYMBOL_COLUMN_TOOLTIP, "SYMBOLS_SYMBOL_COLUMN_TOOLTIP", "tooltip"}, {SYMBOLS_SYMBOL_COLUMN_ID, "SYMBOLS_SYMBOL_COLUMN_ID", "id"}, {SYMBOLS_SYMBOL_COLUMN_PACKAGE, "SYMBOLS_SYMBOL_COLUMN_PACKAGE", "package"}, {SYMBOLS_SYMBOL_COLUMN_N_COLUMNS, "SYMBOLS_SYMBOL_COLUMN_N_COLUMNS", "n-columns"}, {0, NULL, NULL}};
		GType symbols_symbol_column_type_id;
		symbols_symbol_column_type_id = g_enum_register_static ("SymbolsSymbolColumn", values);
		g_once_init_leave (&symbols_symbol_column_type_id__volatile, symbols_symbol_column_type_id);
	}
	return symbols_symbol_column_type_id__volatile;
}


static GType symbols_category_column_get_type (void) {
	static volatile gsize symbols_category_column_type_id__volatile = 0;
	if (g_once_init_enter (&symbols_category_column_type_id__volatile)) {
		static const GEnumValue values[] = {{SYMBOLS_CATEGORY_COLUMN_ICON, "SYMBOLS_CATEGORY_COLUMN_ICON", "icon"}, {SYMBOLS_CATEGORY_COLUMN_NAME, "SYMBOLS_CATEGORY_COLUMN_NAME", "name"}, {SYMBOLS_CATEGORY_COLUMN_N_COLUMNS, "SYMBOLS_CATEGORY_COLUMN_N_COLUMNS", "n-columns"}, {0, NULL, NULL}};
		GType symbols_category_column_type_id;
		symbols_category_column_type_id = g_enum_register_static ("SymbolsCategoryColumn", values);
		g_once_init_leave (&symbols_category_column_type_id__volatile, symbols_category_column_type_id);
	}
	return symbols_category_column_type_id__volatile;
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


Symbols* symbols_construct (GType object_type, MainWindow* main_window) {
	Symbols * self;
	GError * _inner_error_ = NULL;
	g_return_val_if_fail (main_window != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	if (!symbols_stores_initialized) {
		GtkListStore* _tmp0_;
		GdkPixbuf* pixbuf;
		GtkTreeIter iter = {0};
		GtkListStore* _tmp2_;
		GtkListStore* _tmp3_;
		GtkListStore* _tmp4_;
		GtkListStore* _tmp5_;
		GtkListStore* _tmp6_;
		GtkListStore* _tmp7_;
		GtkListStore* _tmp8_;
		GtkListStore* _tmp9_;
		GtkListStore* _tmp10_;
		symbols_categories_store = (_tmp0_ = gtk_list_store_new ((gint) SYMBOLS_CATEGORY_COLUMN_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING), _g_object_unref0 (symbols_categories_store), _tmp0_);
		{
			SymbolsCategoryInfo* info_collection;
			int info_collection_length1;
			int info_it;
			info_collection = SYMBOLS_categories;
			info_collection_length1 = G_N_ELEMENTS (SYMBOLS_categories);
			for (info_it = 0; info_it < G_N_ELEMENTS (SYMBOLS_categories); info_it = info_it + 1) {
				SymbolsCategoryInfo _tmp1_ = {0};
				SymbolsCategoryInfo info;
				info = (symbols_category_info_copy (&info_collection[info_it], &_tmp1_), _tmp1_);
				{
					{
						GdkPixbuf* pixbuf;
						GtkTreeIter iter = {0};
						pixbuf = gdk_pixbuf_new_from_file (info.icon, &_inner_error_);
						if (_inner_error_ != NULL) {
							goto __catch1_g_error;
						}
						gtk_list_store_append (symbols_categories_store, &iter);
						gtk_list_store_set (symbols_categories_store, &iter, SYMBOLS_CATEGORY_COLUMN_ICON, pixbuf, SYMBOLS_CATEGORY_COLUMN_NAME, _ (info.name), -1, -1);
						_g_object_unref0 (pixbuf);
					}
					goto __finally1;
					__catch1_g_error:
					{
						GError * e;
						e = _inner_error_;
						_inner_error_ = NULL;
						{
							fprintf (stderr, "Warning: impossible to load the symbol: %s\n", e->message);
							_g_error_free0 (e);
							symbols_category_info_destroy (&info);
							continue;
						}
					}
					__finally1:
					if (_inner_error_ != NULL) {
						symbols_category_info_destroy (&info);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
						g_clear_error (&_inner_error_);
						return NULL;
					}
					symbols_category_info_destroy (&info);
				}
			}
		}
		pixbuf = utils_get_pixbuf_from_stock (GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU);
		gtk_list_store_append (symbols_categories_store, &iter);
		gtk_list_store_set (symbols_categories_store, &iter, SYMBOLS_CATEGORY_COLUMN_ICON, pixbuf, SYMBOLS_CATEGORY_COLUMN_NAME, _ ("Most Used"), -1, -1);
		symbols_symbols_stores[0] = (_tmp2_ = symbols_get_symbol_store (self, SYMBOLS_symbols_greek, G_N_ELEMENTS (SYMBOLS_symbols_greek)), _g_object_unref0 (symbols_symbols_stores[0]), _tmp2_);
		symbols_symbols_stores[1] = (_tmp3_ = symbols_get_symbol_store (self, SYMBOLS_symbols_arrows, G_N_ELEMENTS (SYMBOLS_symbols_arrows)), _g_object_unref0 (symbols_symbols_stores[1]), _tmp3_);
		symbols_symbols_stores[2] = (_tmp4_ = symbols_get_symbol_store (self, SYMBOLS_symbols_relations, G_N_ELEMENTS (SYMBOLS_symbols_relations)), _g_object_unref0 (symbols_symbols_stores[2]), _tmp4_);
		symbols_symbols_stores[3] = (_tmp5_ = symbols_get_symbol_store (self, SYMBOLS_symbols_operators, G_N_ELEMENTS (SYMBOLS_symbols_operators)), _g_object_unref0 (symbols_symbols_stores[3]), _tmp5_);
		symbols_symbols_stores[4] = (_tmp6_ = symbols_get_symbol_store (self, SYMBOLS_symbols_delimiters, G_N_ELEMENTS (SYMBOLS_symbols_delimiters)), _g_object_unref0 (symbols_symbols_stores[4]), _tmp6_);
		symbols_symbols_stores[5] = (_tmp7_ = symbols_get_symbol_store (self, SYMBOLS_symbols_misc_math, G_N_ELEMENTS (SYMBOLS_symbols_misc_math)), _g_object_unref0 (symbols_symbols_stores[5]), _tmp7_);
		symbols_symbols_stores[6] = (_tmp8_ = symbols_get_symbol_store (self, SYMBOLS_symbols_misc_text, G_N_ELEMENTS (SYMBOLS_symbols_misc_text)), _g_object_unref0 (symbols_symbols_stores[6]), _tmp8_);
		symbols_symbols_stores[7] = (_tmp10_ = _g_object_ref0 (symbols_mus_store = (_tmp9_ = gtk_list_store_new ((gint) SYMBOLS_SYMBOL_COLUMN_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING), _g_object_unref0 (symbols_mus_store), _tmp9_)), _g_object_unref0 (symbols_symbols_stores[7]), _tmp10_);
		symbols_reload_most_used_symbols ();
		symbols_stores_initialized = TRUE;
		_g_object_unref0 (pixbuf);
	}
	self->priv->main_window = main_window;
	symbols_create_icon_views (self);
	return self;
}


Symbols* symbols_new (MainWindow* main_window) {
	return symbols_construct (TYPE_SYMBOLS, main_window);
}


static void _lambda48_ (Symbols* self) {
	AppSettings* _tmp0_;
	gtk_list_store_clear (symbols_mus_store);
	app_settings_clear_most_used_symbols (_tmp0_ = app_settings_get_default ());
	_g_object_unref0 (_tmp0_);
}


static void __lambda48__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	_lambda48_ (self);
}


static gpointer _gtk_tree_path_copy0 (gpointer self) {
	return self ? gtk_tree_path_copy (self) : NULL;
}


static void _g_list_free_gtk_tree_path_free (GList* self) {
	g_list_foreach (self, (GFunc) gtk_tree_path_free, NULL);
	g_list_free (self);
}


static void _lambda49_ (Block3Data* _data3_) {
	Symbols * self;
	GList* selected_items;
	GtkTreePath* path;
	self = _data3_->self;
	selected_items = gtk_icon_view_get_selected_items (_data3_->categories_view);
	path = _gtk_tree_path_copy0 ((GtkTreePath*) g_list_nth_data (selected_items, (guint) 0));
	if (path != NULL) {
		gint num;
		num = gtk_tree_path_get_indices (path)[0];
		gtk_icon_view_set_model (_data3_->symbol_view, (GtkTreeModel*) symbols_symbols_stores[num]);
		if (num == (symbols_symbols_stores_length1 - 1)) {
			gtk_widget_show ((GtkWidget*) _data3_->button);
		} else {
			gtk_widget_hide ((GtkWidget*) _data3_->button);
		}
	}
	_gtk_tree_path_free0 (path);
	__g_list_free_gtk_tree_path_free0 (selected_items);
}


static void __lambda49__gtk_icon_view_selection_changed (GtkIconView* _sender, gpointer self) {
	_lambda49_ (self);
}


static void _lambda50_ (Block3Data* _data3_) {
	Symbols * self;
	GList* selected_items;
	GtkTreePath* path;
	GtkTreeModel* model;
	GtkTreeIter _tmp0_ = {0};
	GtkTreeIter iter;
	gboolean _tmp1_ = FALSE;
	self = _data3_->self;
	if (main_window_get_active_tab (self->priv->main_window) == NULL) {
		g_signal_emit_by_name (_data3_->symbol_view, "unselect-all");
		return;
	}
	selected_items = gtk_icon_view_get_selected_items (_data3_->symbol_view);
	g_signal_emit_by_name (_data3_->symbol_view, "unselect-all");
	path = _gtk_tree_path_copy0 ((GtkTreePath*) g_list_nth_data (selected_items, (guint) 0));
	model = _g_object_ref0 (gtk_icon_view_get_model (_data3_->symbol_view));
	iter = (_tmp0_);
	if (path != NULL) {
		_tmp1_ = gtk_tree_model_get_iter (model, &iter, path);
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		char* latex_command;
		char* id;
		char* package;
		const char* _tmp2_;
		AppSettings* _tmp3_;
		latex_command = NULL;
		id = NULL;
		package = NULL;
		gtk_tree_model_get (model, &iter, SYMBOLS_SYMBOL_COLUMN_COMMAND, &latex_command, SYMBOLS_SYMBOL_COLUMN_ID, &id, SYMBOLS_SYMBOL_COLUMN_PACKAGE, &package, -1, -1);
		gtk_text_buffer_begin_user_action ((GtkTextBuffer*) main_window_get_active_document (self->priv->main_window));
		gtk_text_buffer_insert_at_cursor ((GtkTextBuffer*) main_window_get_active_document (self->priv->main_window), latex_command, -1);
		gtk_text_buffer_insert_at_cursor ((GtkTextBuffer*) main_window_get_active_document (self->priv->main_window), " ", -1);
		gtk_text_buffer_end_user_action ((GtkTextBuffer*) main_window_get_active_document (self->priv->main_window));
		gtk_widget_grab_focus ((GtkWidget*) main_window_get_active_view (self->priv->main_window));
		_tmp2_ = NULL;
		if (_vala_strcmp0 (package, "") != 0) {
			_tmp2_ = package;
		} else {
			_tmp2_ = NULL;
		}
		app_settings_add_symbol (_tmp3_ = app_settings_get_default (), id, latex_command, _tmp2_);
		_g_object_unref0 (_tmp3_);
		_g_free0 (package);
		_g_free0 (id);
		_g_free0 (latex_command);
	}
	_g_object_unref0 (model);
	_gtk_tree_path_free0 (path);
	__g_list_free_gtk_tree_path_free0 (selected_items);
}


static void __lambda50__gtk_icon_view_selection_changed (GtkIconView* _sender, gpointer self) {
	_lambda50_ (self);
}


static Block3Data* block3_data_ref (Block3Data* _data3_) {
	g_atomic_int_inc (&_data3_->_ref_count_);
	return _data3_;
}


static void block3_data_unref (Block3Data* _data3_) {
	if (g_atomic_int_dec_and_test (&_data3_->_ref_count_)) {
		_g_object_unref0 (_data3_->self);
		_g_object_unref0 (_data3_->button);
		_g_object_unref0 (_data3_->symbol_view);
		_g_object_unref0 (_data3_->categories_view);
		g_slice_free (Block3Data, _data3_);
	}
}


static void symbols_create_icon_views (Symbols* self) {
	Block3Data* _data3_;
	GtkWidget* sw;
	g_return_if_fail (self != NULL);
	_data3_ = g_slice_new0 (Block3Data);
	_data3_->_ref_count_ = 1;
	_data3_->self = g_object_ref (self);
	_data3_->categories_view = g_object_ref_sink ((GtkIconView*) gtk_icon_view_new_with_model ((GtkTreeModel*) symbols_categories_store));
	gtk_icon_view_set_pixbuf_column (_data3_->categories_view, (gint) SYMBOLS_CATEGORY_COLUMN_ICON);
	gtk_icon_view_set_text_column (_data3_->categories_view, (gint) SYMBOLS_CATEGORY_COLUMN_NAME);
	gtk_icon_view_set_selection_mode (_data3_->categories_view, GTK_SELECTION_SINGLE);
	gtk_icon_view_set_orientation (_data3_->categories_view, GTK_ORIENTATION_HORIZONTAL);
	gtk_icon_view_set_spacing (_data3_->categories_view, 5);
	gtk_icon_view_set_row_spacing (_data3_->categories_view, 0);
	gtk_icon_view_set_column_spacing (_data3_->categories_view, 0);
	gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) _data3_->categories_view, FALSE, FALSE, (guint) 0);
	gtk_widget_show ((GtkWidget*) _data3_->categories_view);
	_data3_->symbol_view = g_object_ref_sink ((GtkIconView*) gtk_icon_view_new_with_model ((GtkTreeModel*) symbols_symbols_stores[0]));
	gtk_icon_view_set_pixbuf_column (_data3_->symbol_view, (gint) SYMBOLS_SYMBOL_COLUMN_PIXBUF);
	gtk_icon_view_set_tooltip_column (_data3_->symbol_view, (gint) SYMBOLS_SYMBOL_COLUMN_TOOLTIP);
	gtk_icon_view_set_selection_mode (_data3_->symbol_view, GTK_SELECTION_SINGLE);
	gtk_icon_view_set_spacing (_data3_->symbol_view, 0);
	gtk_icon_view_set_row_spacing (_data3_->symbol_view, 0);
	gtk_icon_view_set_column_spacing (_data3_->symbol_view, 0);
	sw = utils_add_scrollbar ((GtkWidget*) _data3_->symbol_view);
	gtk_box_pack_start ((GtkBox*) self, sw, TRUE, TRUE, 0);
	gtk_widget_show_all (sw);
	_data3_->button = g_object_ref_sink ((GtkButton*) gtk_button_new_from_stock (GTK_STOCK_CLEAR));
	gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) _data3_->button, FALSE, FALSE, (guint) 2);
	g_signal_connect_object (_data3_->button, "clicked", (GCallback) __lambda48__gtk_button_clicked, self, 0);
	g_signal_connect_data (_data3_->categories_view, "selection-changed", (GCallback) __lambda49__gtk_icon_view_selection_changed, block3_data_ref (_data3_), (GClosureNotify) block3_data_unref, 0);
	g_signal_connect_data (_data3_->symbol_view, "selection-changed", (GCallback) __lambda50__gtk_icon_view_selection_changed, block3_data_ref (_data3_), (GClosureNotify) block3_data_unref, 0);
	_g_object_unref0 (sw);
	block3_data_unref (_data3_);
}


static GtkListStore* symbols_get_symbol_store (Symbols* self, SymbolsSymbolInfo* symbols, int symbols_length1) {
	GtkListStore* result = NULL;
	GtkListStore* symbol_store;
	g_return_val_if_fail (self != NULL, NULL);
	symbol_store = gtk_list_store_new ((gint) SYMBOLS_SYMBOL_COLUMN_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	{
		SymbolsSymbolInfo* symbol_collection;
		int symbol_collection_length1;
		int symbol_it;
		symbol_collection = symbols;
		symbol_collection_length1 = symbols_length1;
		for (symbol_it = 0; symbol_it < symbols_length1; symbol_it = symbol_it + 1) {
			SymbolsSymbolInfo _tmp0_ = {0};
			SymbolsSymbolInfo symbol;
			symbol = (symbols_symbol_info_copy (&symbol_collection[symbol_it], &_tmp0_), _tmp0_);
			{
				symbols_insert_symbol (symbol_store, -1, &symbol);
				symbols_symbol_info_destroy (&symbol);
			}
		}
	}
	result = symbol_store;
	return result;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, (gssize) (-1));
	return result;
}


static char* string_slice (const char* self, glong start, glong end) {
	char* result = NULL;
	glong string_length;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	const char* start_string;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = string_get_length (self);
	if (start < 0) {
		start = string_length + start;
	}
	if (end < 0) {
		end = string_length + end;
	}
	if (start >= 0) {
		_tmp0_ = start <= string_length;
	} else {
		_tmp0_ = FALSE;
	}
	g_return_val_if_fail (_tmp0_, NULL);
	if (end >= 0) {
		_tmp1_ = end <= string_length;
	} else {
		_tmp1_ = FALSE;
	}
	g_return_val_if_fail (_tmp1_, NULL);
	g_return_val_if_fail (start <= end, NULL);
	start_string = g_utf8_offset_to_pointer (self, start);
	result = g_strndup (start_string, ((gchar*) g_utf8_offset_to_pointer (start_string, end - start)) - ((gchar*) start_string));
	return result;
}


static char* symbols_get_symbol_id (const char* path) {
	char* result = NULL;
	glong l;
	gboolean second;
	g_return_val_if_fail (path != NULL, NULL);
	l = string_get_length (path);
	second = FALSE;
	{
		glong i;
		i = l - 1;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
			while (TRUE) {
				if (!_tmp0_) {
					i--;
				}
				_tmp0_ = FALSE;
				if (!(i >= 0)) {
					break;
				}
				if (g_utf8_get_char (g_utf8_offset_to_pointer (path, i)) == '/') {
					if (second) {
						result = string_slice (path, i + 1, l);
						return result;
					}
					second = TRUE;
				}
			}
		}
	}
	g_return_val_if_reached ("no id");
}


void symbols_reload_most_used_symbols (void) {
	AppSettings* _tmp0_;
	GeeList* _tmp1_;
	GeeList* most_used_symbols;
	most_used_symbols = (_tmp1_ = app_settings_get_most_used_symbols (_tmp0_ = app_settings_get_default ()), _g_object_unref0 (_tmp0_), _tmp1_);
	gtk_list_store_clear (symbols_mus_store);
	{
		GeeIterator* _mus_it;
		_mus_it = gee_iterable_iterator ((GeeIterable*) most_used_symbols);
		while (TRUE) {
			MostUsedSymbol* _tmp2_;
			MostUsedSymbol _tmp3_ = {0};
			MostUsedSymbol _tmp4_;
			MostUsedSymbol mus;
			SymbolsSymbolInfo _tmp5_ = {0};
			SymbolsSymbolInfo symbol;
			if (!gee_iterator_next (_mus_it)) {
				break;
			}
			mus = (_tmp4_ = (most_used_symbol_copy (_tmp2_ = (MostUsedSymbol*) gee_iterator_get (_mus_it), &_tmp3_), _tmp3_), _most_used_symbol_free0 (_tmp2_), _tmp4_);
			symbol = (symbols_get_symbol_info_from_most_used (&mus, &_tmp5_), _tmp5_);
			symbols_insert_symbol (symbols_mus_store, -1, &symbol);
			symbols_symbol_info_destroy (&symbol);
			most_used_symbol_destroy (&mus);
		}
		_g_object_unref0 (_mus_it);
	}
	_g_object_unref0 (most_used_symbols);
}


static void symbols_get_symbol_info_from_most_used (MostUsedSymbol* mus, SymbolsSymbolInfo* result) {
	SymbolsSymbolInfo symbol = {0};
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	memset (&symbol, 0, sizeof (SymbolsSymbolInfo));
	symbol.filename = (_tmp0_ = g_strconcat (DATA_DIR "/images/", (*mus).id, NULL), _g_free0 (symbol.filename), _tmp0_);
	symbol.latex_command = (_tmp1_ = g_strdup ((*mus).latex_command), _g_free0 (symbol.latex_command), _tmp1_);
	symbol.package_required = (_tmp2_ = g_strdup ((*mus).package_required), _g_free0 (symbol.package_required), _tmp2_);
	*result = symbol;
	return;
}


static void symbols_insert_symbol (GtkListStore* symbol_store, gint index, SymbolsSymbolInfo* symbol) {
	GError * _inner_error_ = NULL;
	g_return_if_fail (symbol_store != NULL);
	{
		GdkPixbuf* pixbuf;
		char* tooltip;
		GtkTreeIter iter = {0};
		char* _tmp2_;
		char* _tmp4_;
		pixbuf = gdk_pixbuf_new_from_file ((*symbol).filename, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch2_g_error;
		}
		tooltip = g_markup_escape_text ((*symbol).latex_command, -1);
		if ((*symbol).package_required != NULL) {
			char* _tmp0_;
			char* _tmp1_;
			tooltip = (_tmp1_ = g_strconcat (tooltip, _tmp0_ = g_strdup_printf (" (package %s)", (*symbol).package_required), NULL), _g_free0 (tooltip), _tmp1_);
			_g_free0 (_tmp0_);
		}
		if (index == (-1)) {
			gtk_list_store_append (symbol_store, &iter);
		} else {
			gtk_list_store_insert (symbol_store, &iter, index);
		}
		_tmp2_ = g_strdup ((*symbol).package_required);
		if (_tmp2_ == NULL) {
			char* _tmp3_;
			_tmp2_ = (_tmp3_ = g_strdup (""), _g_free0 (_tmp2_), _tmp3_);
		}
		gtk_list_store_set (symbol_store, &iter, SYMBOLS_SYMBOL_COLUMN_PIXBUF, pixbuf, SYMBOLS_SYMBOL_COLUMN_COMMAND, (*symbol).latex_command, SYMBOLS_SYMBOL_COLUMN_TOOLTIP, tooltip, SYMBOLS_SYMBOL_COLUMN_ID, _tmp4_ = symbols_get_symbol_id ((*symbol).filename), SYMBOLS_SYMBOL_COLUMN_PACKAGE, _tmp2_, -1, -1);
		_g_free0 (_tmp4_);
		_g_free0 (_tmp2_);
		_g_free0 (tooltip);
		_g_object_unref0 (pixbuf);
	}
	goto __finally2;
	__catch2_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			fprintf (stderr, "Warning: impossible to load the symbol: %s\n", e->message);
			_g_error_free0 (e);
		}
	}
	__finally2:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
		g_clear_error (&_inner_error_);
		return;
	}
}


void symbols_insert_most_used_symbol (gint index, MostUsedSymbol* symbol) {
	SymbolsSymbolInfo _tmp0_ = {0};
	SymbolsSymbolInfo _tmp1_;
	SymbolsSymbolInfo _tmp2_;
	symbols_insert_symbol (symbols_mus_store, index, (_tmp2_ = _tmp1_ = (symbols_get_symbol_info_from_most_used (symbol, &_tmp0_), _tmp0_), &_tmp2_));
	symbols_symbol_info_destroy (&_tmp1_);
}


void symbols_remove_most_used_symbol (gint index) {
	GtkTreePath* path;
	GtkTreeIter iter = {0};
	path = gtk_tree_path_new_from_indices (index, -1, -1);
	if (gtk_tree_model_get_iter ((GtkTreeModel*) symbols_mus_store, &iter, path)) {
		gtk_list_store_remove (symbols_mus_store, &iter);
	}
	_gtk_tree_path_free0 (path);
}


void symbols_swap_most_used_symbol (gint current_index, gint new_index) {
	GtkTreePath* current_path;
	GtkTreePath* new_path;
	GtkTreeIter _tmp0_ = {0};
	GtkTreeIter current_iter;
	GtkTreeIter _tmp1_ = {0};
	GtkTreeIter new_iter;
	gboolean _tmp2_ = FALSE;
	current_path = gtk_tree_path_new_from_indices (current_index, -1, -1);
	new_path = gtk_tree_path_new_from_indices (new_index, -1, -1);
	current_iter = (_tmp0_);
	new_iter = (_tmp1_);
	if (gtk_tree_model_get_iter ((GtkTreeModel*) symbols_mus_store, &current_iter, current_path)) {
		_tmp2_ = gtk_tree_model_get_iter ((GtkTreeModel*) symbols_mus_store, &new_iter, new_path);
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		gtk_list_store_move_before (symbols_mus_store, &current_iter, &new_iter);
	}
	_gtk_tree_path_free0 (new_path);
	_gtk_tree_path_free0 (current_path);
}


static void symbols_symbol_info_copy (const SymbolsSymbolInfo* self, SymbolsSymbolInfo* dest) {
	dest->filename = g_strdup (self->filename);
	dest->latex_command = g_strdup (self->latex_command);
	dest->package_required = g_strdup (self->package_required);
}


static void symbols_symbol_info_destroy (SymbolsSymbolInfo* self) {
	_g_free0 (self->filename);
	_g_free0 (self->latex_command);
	_g_free0 (self->package_required);
}


static SymbolsSymbolInfo* symbols_symbol_info_dup (const SymbolsSymbolInfo* self) {
	SymbolsSymbolInfo* dup;
	dup = g_new0 (SymbolsSymbolInfo, 1);
	symbols_symbol_info_copy (self, dup);
	return dup;
}


static void symbols_symbol_info_free (SymbolsSymbolInfo* self) {
	symbols_symbol_info_destroy (self);
	g_free (self);
}


static GType symbols_symbol_info_get_type (void) {
	static volatile gsize symbols_symbol_info_type_id__volatile = 0;
	if (g_once_init_enter (&symbols_symbol_info_type_id__volatile)) {
		GType symbols_symbol_info_type_id;
		symbols_symbol_info_type_id = g_boxed_type_register_static ("SymbolsSymbolInfo", (GBoxedCopyFunc) symbols_symbol_info_dup, (GBoxedFreeFunc) symbols_symbol_info_free);
		g_once_init_leave (&symbols_symbol_info_type_id__volatile, symbols_symbol_info_type_id);
	}
	return symbols_symbol_info_type_id__volatile;
}


static void symbols_category_info_copy (const SymbolsCategoryInfo* self, SymbolsCategoryInfo* dest) {
	dest->name = g_strdup (self->name);
	dest->icon = g_strdup (self->icon);
}


static void symbols_category_info_destroy (SymbolsCategoryInfo* self) {
	_g_free0 (self->name);
	_g_free0 (self->icon);
}


static SymbolsCategoryInfo* symbols_category_info_dup (const SymbolsCategoryInfo* self) {
	SymbolsCategoryInfo* dup;
	dup = g_new0 (SymbolsCategoryInfo, 1);
	symbols_category_info_copy (self, dup);
	return dup;
}


static void symbols_category_info_free (SymbolsCategoryInfo* self) {
	symbols_category_info_destroy (self);
	g_free (self);
}


static GType symbols_category_info_get_type (void) {
	static volatile gsize symbols_category_info_type_id__volatile = 0;
	if (g_once_init_enter (&symbols_category_info_type_id__volatile)) {
		GType symbols_category_info_type_id;
		symbols_category_info_type_id = g_boxed_type_register_static ("SymbolsCategoryInfo", (GBoxedCopyFunc) symbols_category_info_dup, (GBoxedFreeFunc) symbols_category_info_free);
		g_once_init_leave (&symbols_category_info_type_id__volatile, symbols_category_info_type_id);
	}
	return symbols_category_info_type_id__volatile;
}


static void symbols_class_init (SymbolsClass * klass) {
	symbols_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (SymbolsPrivate));
	G_OBJECT_CLASS (klass)->finalize = symbols_finalize;
	symbols_symbols_stores = g_new0 (GtkListStore*, 8 + 1);
	symbols_symbols_stores_length1 = 8;
}


static void symbols_instance_init (Symbols * self) {
	self->priv = SYMBOLS_GET_PRIVATE (self);
}


static void symbols_finalize (GObject* obj) {
	Symbols * self;
	self = SYMBOLS (obj);
	G_OBJECT_CLASS (symbols_parent_class)->finalize (obj);
}


GType symbols_get_type (void) {
	static volatile gsize symbols_type_id__volatile = 0;
	if (g_once_init_enter (&symbols_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SymbolsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) symbols_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Symbols), 0, (GInstanceInitFunc) symbols_instance_init, NULL };
		GType symbols_type_id;
		symbols_type_id = g_type_register_static (GTK_TYPE_VBOX, "Symbols", &g_define_type_info, 0);
		g_once_init_leave (&symbols_type_id__volatile, symbols_type_id);
	}
	return symbols_type_id__volatile;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




